@@ -1,5 +1,5 @@
 #
-# $Id: Agent.pm,v 0.1.1.9 2000/10/01 20:02:29 ram Exp $
+# $Id: Agent.pm,v 0.2 2000/11/06 19:30:32 ram Exp $
 #
 #  Copyright (c) 1999, Raphael Manfredi
 #  
@@ -8,39 +8,8 @@
 #
 # HISTORY
 # $Log: Agent.pm,v $
-# Revision 0.1.1.9  2000/10/01 20:02:29  ram
-# patch8: changed version number
-#
-# Revision 0.1.1.8  2000/10/01 19:49:22  ram
-# patch8: added support for Devel::Datum
-# patch8: new logcarp, logxcarp and logxcroak interface
-#
-# Revision 0.1.1.7  2000/07/04 20:02:59  ram
-# patch7: forgot to increase version number at last patch
-#
-# Revision 0.1.1.6  2000/07/04 20:00:55  ram
-# patch6: was missing default init check before calling logwrite()
-#
-# Revision 0.1.1.5  2000/06/20 21:21:45  ram
-# patch5: added logcroak()
-# patch5: new logwrite() routine for upper-level apps
-# patch5: fixed arg processing to avoid dying on 2nd calls to logconfig()
-# patch5: fixed typos in debug init and man page
-#
-# Revision 0.1.1.4  2000/03/30 19:24:04  ram
-# patch4: updated version number
-#
-# Revision 0.1.1.3  2000/03/05 22:20:48  ram
-# patch3: updated version number
-#
-# Revision 0.1.1.2  1999/12/09 17:30:40  ram
-# patch2: wrote two versions of format_args and eval proper one
-#
-# Revision 0.1.1.1  1999/12/08 21:51:38  ram
-# patch1: forgot that /(?<!)/ is a 5.005 feature
-#
-# Revision 0.1  1999/12/07 21:09:44  ram
-# Baseline for first alpha release.
+# Revision 0.2  2000/11/06 19:30:32  ram
+# Baseline for second Alpha release.
 #
 # $EndLog$
 #
@@ -65,106 +34,12 @@ use vars qw(@ISA @EXPORT @EXPORT_OK);
 	logwrite
 );
 
-require Log::Agent::Message;
+use Log::Agent::Priorities qw(:LEVELS priority_level level_from_prio);
+use Log::Agent::Formatting qw(caller_format_args);
 
-$VERSION = '0.108';
+$VERSION = '0.200';
 
-###
-### Utilities
-###
-### (not autoloaded due to use of different versions)
-###
-
-#
-# format_args
-#
-# Format arguments using sprintf() if there is more than one, taking the
-# first as the format. Otherwise, we take only its first and only argument.
-#
-# Returns a Log::Agent::Message object, which, when stringified, prints
-# the string itself.
-#
-# We process syslog's %m macro as being the current error message ($!) in
-# the first argument only. Doing it at this level means it will be supported
-# independently from the driver they'll choose. It's also done BEFORE any
-# log-related system call, thus ensuring that $! retains its original value.
-#
-# If caller information is wanted, add it to the log message according to
-# the specifications chosen at logconfig() time.
-#
-if ($] >= 5.005) { eval q{				# if VERSION >= 5.005
-
-# 5.005 and later version grok /(?<!)/
-sub format_args {
-	my $fmt = shift;
-	$fmt =~ s/((?<!%)(?:%%)*)%m/$!/g;
-	my $str = Log::Agent::Message->make(@_ ? sprintf($fmt, @_) : $fmt);
-	$Caller->insert($str) if $Caller;
-	return $str;
-}
-
-}} else { eval q{						# else /* VERSION < 5.005 */
-
-# pre-5.005 does not grok /(?<!)/
-sub format_args {
-	my $fmt = shift;
-	$fmt =~ s/%%/\01/g;
-	$fmt =~ s/%m/$!/g;
-	$fmt =~ s/\01/%%/g;
-	my $str = Log::Agent::Message->make(@_ ? sprintf($fmt, @_) : $fmt);
-	$Caller->insert($str) if $Caller;
-	return $str;
-}
-
-}}										# endif /* VERSION >= 5.005 */
-
-###
-### Priority/level mappings
-###
-### (not autoloaded due to the use of lexical variables)
-###
-
-my @basic_prio = qw(
-	alert critical
-	error error
-	warning warning
-	notice notice
-	info info);
-
-#
-# prio_from_level
-#
-# Given a level, compute suitable priority.
-#
-sub prio_from_level {
-	my ($level) = @_;
-	return 'emergency' if $level < 0;
-	return 'debug' if $level > @basic_prio;
-	return $basic_prio[$level];
-}
-
-my %basic_level = (
-	'em'	=> 1,		# emergency
-	'al'	=> 1,		# alert
-	'cr'	=> 1,		# critical
-	'er'	=> 2,		# error
-	'wa'	=> 4,		# warning
-	'no'	=> 6,		# notice
-	'in'	=> 8,		# info
-	'de'	=> 10,		# debug
-);
-
-#
-# level_from_prio
-#
-# Given a syslog priority, compute suitable level.
-#
-sub level_from_prio {
-	my ($prio) = @_;
-	my $canonical = lc(substr($prio, 0, 2));
-	return 10 unless exists $basic_level{$canonical};
-	return $basic_level{$canonical};
-}
+$Trace = NOTICE;	# Default tracing
 
 1;
 __END__
@@ -285,8 +160,8 @@ sub log_default {
 # Die with a full stack trace
 #
 sub logconfess {
-	my $str = format_args(@_);
-	&log_default;
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logconfess($str);
 	bug("back from logconfess in driver $Driver\n");
 }
@@ -299,8 +174,8 @@ sub logconfess {
 #
 sub logcroak {
 	goto &logconfess if $Confess;		# Redirected when -confess
-	my $str = format_args(@_);
-	&log_default;
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logxcroak(0, $str);
 	bug("back from logxcroak in driver $Driver\n");
 }
@@ -313,8 +188,8 @@ sub logcroak {
 sub logxcroak {
 	my $offset = shift;
 	goto &logconfess if $Confess;		# Redirected when -confess
-	my $str = format_args(@_);
-	&log_default;
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logxcroak($offset, $str);
 	bug("back from logxcroak in driver $Driver\n");
 }
@@ -327,8 +202,8 @@ sub logxcroak {
 #
 sub logdie {
 	goto &logconfess if $Confess;		# Redirected when -confess
-	my $str = format_args(@_);
-	&log_default;
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logdie($str);
 	bug("back from logdie in driver $Driver\n");
 }
@@ -336,22 +211,24 @@ sub logdie {
 #
 # logerr
 #
-# Log error
+# Log error, at the "error" level.
 #
 sub logerr {
-	my $str = format_args(@_);
-	&log_default;
+	return if $Trace < ERROR;
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logerr($str);
 }
 
 #
 # logcarp
 #
-# Warning, from the perspective of our caller
+# Warning, from the perspective of our caller (at the "warning" level)
 #
 sub logcarp {
-	my $str = format_args(@_);
-	&log_default;
+	return if $Trace < WARN;
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logxcarp(0, $str);
 }
 
@@ -361,31 +238,34 @@ sub logcarp {
 # Same a logcarp, but with a specific additional offset.
 #
 sub logxcarp {
+	return if $Trace < WARN;
 	my $offset = shift;
-	my $str = format_args(@_);
-	&log_default;
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logxcarp($offset, $str);
 }
 
 #
 # logwarn
 #
-# Log warning
+# Log warning at the "warning" level.
 #
 sub logwarn {
-	my $str = format_args(@_);
-	&log_default;
+	return if $Trace < WARN;
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logwarn($str);
 }
 
 #
 # logsay
 #
-# Unconditionally log message
+# Log message at the "notice" level.
 #
 sub logsay {
-	my $str = format_args(@_);
-	&log_default;
+	return if $Trace < NOTICE;
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logsay($str);
 }
 
@@ -396,11 +276,11 @@ sub logsay {
 # Trace level must either be a single digit or "priority" or "priority:digit".
 #
 sub logtrc {
-	my ($id, @args) = @_;
-	my ($prio, $level) = priority($id);
-	return if !defined($Trace) || $level > $Trace;
-	my $str = format_args(@args);
-	&log_default;
+	my $id = shift;
+	my ($prio, $level) = priority_level($id);
+	return if $level > $Trace;
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logwrite('output', $prio, $level, $str);
 }
 
@@ -411,11 +291,11 @@ sub logtrc {
 # Debug level must either be a single digit or "priority" or "priority:digit".
 #
 sub logdbg {
-	my ($id, @args) = @_;
-	my ($prio, $level) = priority($id);
+	my $id = shift;
+	my ($prio, $level) = priority_level($id);
 	return if !defined($Debug) || $level > $Debug;
-	my $str = format_args(@args);
-	&log_default;
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logwrite('debug', $prio, $level, $str);
 }
 
@@ -427,37 +307,22 @@ sub logdbg {
 # logwrite		-- not exported by default
 #
 # Write message to the specified channel, at the given priority.
-# This routine is used by both logtrc() and logdbg() and may be redefined.
 #
 sub logwrite {
-	my ($channel, $id, @args) = @_;
-	my ($prio, $level) = priority($id);
-	my $str = format_args(@args);
-	&log_default;
+	my ($channel, $id) = splice(@_, 0, 2);
+	my ($prio, $level) = priority_level($id);
+	my $str = caller_format_args($Caller, \@_);
+	&log_default unless defined $Driver;
 	$Driver->logwrite($channel, $prio, $level, $str);
 }
 
-#
-# priority
-#
-# Decompiles priority which can be either a single digit, a "priority" string
-# or a "priority:digit" string. Returns the priority (computed if none) and
-# the level (computed if none).
-#
-sub priority {
-	my ($id) = @_;
-	return (prio_from_level($id), $id) if $id =~ /^\d+$/;
-	return ($1, $2) if $id =~ /^([^:]+):(\d+)$/;
-	return ($id, level_from_prio($id));
-}
-
 #
 # bug
 #
 # Log bug, and die.
 #
 sub bug {
-	my $str = format_args(@_);
+	my $str = caller_format_args($Caller, @_);
 	logerr("BUG: $str");
 	die "${Prefix}: $str\n";
 }
@@ -492,80 +357,66 @@ Log::Agent - logging agent
 
 =head1 DESCRIPTION
 
-The Log::Agent module provides an abstract layer for logging and tracing,
-which is independant from the actual method used to physically perform
-those activities. It acts as an agent (hence the name) that collects
-the requests and delegates processing to a sublayer: the logging driver.
+The C<Log::Agent> module provides an abstract layer for logging and
+tracing, which is independant from the actual method used to physically
+perform those activities. It acts as an agent (hence the name) that
+collects the requests and delegates processing to a sublayer: the
+logging driver.
 
-The Log::Agent module is meant to be used in all reusable components,
-since they cannot know in advance how the application which ends up
-using them will perform its logging activities: either by emitting
-messages on stdout and errors on stderr, or by directing messages
-to logfiles, or by using syslog(3).
+The C<Log::Agent> module is meant to be used in all reusable components,
+since they cannot know in advance how the application which ends up using
+them will perform its logging activities: either by emitting messages
+on stdout and errors on stderr, or by directing messages to logfiles,
+or by using syslog(3).
 
 The logging interface is common for all the logging drivers, and is
-therefore the result of a compromise between many logging schemes:
-any information given at this level must be either handled by all
-drivers, or may be ignored depending on the application's final choice.
+therefore the result of a compromise between many logging schemes: any
+information given at this level must be either handled by all drivers,
+or may be ignored depending on the application's final choice.
 
-WARNING: THIS INTERFACE IS STILL VERY ALPHA AND IS SUBJECT TO CHANGE
+WARNING: THIS INTERFACE IS STILL SOMEWHAT ALPHA AND COULD STILL CHANGE
 DEPENDING ON THE FEEDBACK I SHALL GET FROM USERS AND FROM MY OWN
 EXPERIENCE USING IT, WITHOUT ANY BACKWARD COMPATIBILITY ASSURANCE.
 
 =head1 PRIORITIES AND LEVEL
 
-The Log::Agent module can use both priorities (as defined by syslog(3))
-or logging levels, or either, in which case there is an implicit
-computation of the missing item. Here are the known priorities (which
-may be abbreviated to the first 2 letters, in a case-insensitive manner)
-and their corresponding logging level:
-
-    emergency   0
-    alert       0
-    critical    1
-    error       2
-    warning     4
-    notice      6
-    info        8
-    debug       10 
+The C<Log::Agent> module can use both priorities (as defined by
+syslog(3)) or logging levels, or either, in which case there is
+an implicit computation of the missing item (i.e. the level 4, for
+instance, corresponds to the "warning" priority, and vice-versa).
+See L<Log::Agent::Priorities> for more details.
 
 A logging level is defined as being a threshold: any level lesser than
 or equal to that threshold will be logged.
 
-Anywhere where a I<priority> is expected, one may specify a number taken
-as a logging level or a string taken as a priority. If the default
-mapping outlined above is not satisfactory, it can be redefined by
-specifying, for instance C<"notice:9">. It will be taken as being of
-level 9, but with a C<notice> priority nonetheless, not C<info> as
-it would have been implicitely determined otherwise.
-
-At the Log::Agent level, it is possible to define a trace level and a
-debug level. Only the messages below those levels (inclusive) will be handed
-out to the underlying driver for logging. They are used by the logtrc()
-and logdbg() routines, respectively.
+At the C<Log::Agent> level, it is possible to define a trace level and
+a debug level. Only the messages below those levels (inclusive) will be
+handed out to the underlying driver for logging. They are used by the
+logtrc() and logdbg() routines, respectively.
 
 =head1 CHANNELS
 
-The Log::Agent class defines three logging channels, which are C<error>,
-C<output> and C<debug>. Depending on the driver used for logging, those
-channels are ignored (typically with syslog()) or may be implicitely defined
-(default logging, i.e. the one achieved by the Log::Agent::Driver::Default
-driver, remaps C<error> to stderr, C<output> and C<debug> to stdout).
+The C<Log::Agent> class defines three logging channels, which are
+C<error>, C<output> and C<debug>. Depending on the driver used for
+logging, those channels are ignored (typically with syslog()) or may
+be implicitely defined (default logging, i.e. the one achieved by the
+C<Log::Agent::Driver::Default> driver, remaps C<error> and C<debug>
+to stderr, C<output> to stdout).
 
 =head1 INTERFACE
 
 Anywhere a I<message> is expected, it can be a single string, or a
-printf()-like format string followed by the required arguments. The special
-macro C<%m> is handled directly by Log::Agent and is replaced by the string
-version of $!, which is the last error message returned by the last failing
-system call.
+printf()-like format string followed by the required arguments. The
+special macro C<%m> is handled directly by C<Log::Agent> and is replaced
+by the string version of $!, which is the last error message returned
+by the last failing system call.
 
-B<NOTE>: There should not be any trailing "\n" in the I<message> strings, nor
-any embededed one, although this is not enforced. Remember that the main
-purpose of Log::Agent is to specify logging messages in a standard way!
-Therefore, most of the time, a "should" should be read as "must" and "should
-not" as "must not", which is the strongest interdiction form available in
-English, as far as I know.
+B<NOTE>: There should not be any trailing "\n" in the I<message> strings,
+nor any embededed one, although this is not enforced. Remember that
+the main purpose of C<Log::Agent> is to specify logging messages in a
+standard way!  Therefore, most of the time, a "should" should be read as
+"must" and "should not" as "must not", which is the strongest interdiction
+form available in English, as far as I know.
 
 Here are valid I<message> examples:
 
@@ -580,32 +431,32 @@ The follwing logging interface is made available to modules:
 =item logdbg I<priority>, I<message>
 
 Debug logging of I<message> to the C<debug> channel.
-You may specify any priority you want, i.e.
-a C<debug> priority is not enforced here. You may even specify C<"notice:4">
-if you wish, to have the message logged if the debug level is set to 4 or less.
-If handed over to syslog(3), the message will nonetheless be logged at the
-C<notice> priority.
+
+You may specify any priority you want, i.e.  a C<debug> priority is
+not enforced here. You may even specify C<"notice:4"> if you wish,
+to have the message logged if the debug level is set to 4 or less.
+If handed over to syslog(3), the message will nonetheless be logged at
+the C<notice> priority.
 
 =item logtrc I<priority>, I<message>
 
 Trace logging of I<message> to the C<output> channel.
-Like logdbg() above, you are not restricted
-to the C<info> priority. This routine checks the logging level (either
-explicit as in C<"info:14"> or implicit as in C<"notice">) against the
-trace level.
+
+Like logdbg() above, you are not restricted to the C<info> priority. This
+routine checks the logging level (either explicit as in C<"info:14">
+or implicit as in C<"notice">) against the trace level.
 
 =item logsay I<message>
 
-Unconditionally log the message at the C<notice> priority to the C<output>
-channel.
-The logging always takes place, but only if the routine is called.
-This means you can still say:
+Log the message at the C<notice> priority to the C<output> channel.
+The logging always takes place under the default C<-trace> settings, but
+only if the routine is called, naturally.  This means you can still say:
 
     logsay "some trace message" if $verbose;
 
-and control whether the message is emitted by using some external configuration
-for your module (e.g. by adding a -verbose flag to the creation routine of
-your class).
+and control whether the message is emitted by using some external
+configuration for your module (e.g. by adding a -verbose flag to the
+creation routine of your class).
 
 =item logwarn I<message>
 
@@ -627,17 +478,17 @@ and then dies.
 
 =item logconfess I<message>
 
-Same as logdie(), but issues a Carp::confess(3) call instead.
-It is possible to configure the Log::Agent module via the C<-confess> switch
-to automatically redirect a logdie() to logconfess(), which is invaluable
-during unit testing.
+Same as logdie(), but issues a Carp::confess(3) call instead.  It is
+possible to configure the C<Log::Agent> module via the C<-confess>
+switch to automatically redirect a logdie() to logconfess(), which is
+invaluable during unit testing.
 
 =item logcroak I<message>
 
-Same as logdie(), but issues a Carp::croak(3) call instead.
-It is possible to configure the Log::Agent module via the C<-confess> switch
-to automatically redirect a logcroak() to logconfess(), which is invaluable
-during unit testing.
+Same as logdie(), but issues a Carp::croak(3) call instead.  It is
+possible to configure the C<Log::Agent> module via the C<-confess>
+switch to automatically redirect a logcroak() to logconfess(), which is
+invaluable during unit testing.
 
 =item Log::Agent::inited
 
@@ -664,10 +515,10 @@ stack.  To report an error one level above your caller, set it to 1.
 
 =back
 
-For applications that wish to implement a debug layer on top of Log::Agent,
-the following routine is provided.  Note that it is not imported by default,
-i.e. it needs to be explicitely mentionned at C<use> time, since it is not
-meant to be used directly under regular usage.
+For applications that wish to implement a debug layer on top of
+C<Log::Agent>, the following routine is provided.  Note that it is not
+imported by default, i.e. it needs to be explicitely mentionned at C<use>
+time, since it is not meant to be used directly under regular usage.
 
 =over
 
@@ -714,7 +565,7 @@ pass through, others will return prematurely without logging anything.
 =item C<-driver> => I<driver_object>
 
 This switch defines the driver object to be used, which must be an heir of
-the Log::Agent::Driver class. See L<Log::Agent::Driver(3)> for a list
+the C<Log::Agent::Driver> class. See L<Log::Agent::Driver(3)> for a list
 of the available drivers.
 
 =item C<-level> => I<priority or level>
@@ -724,27 +575,31 @@ common value.
 
 =item C<-prefix> => I<name>
 
-Defines the application name which will be pre-pended to all messages, followed
-by C<": "> (a colon and a space). Using this switch alone will configure
-the default driver to use that prefix (stripped down to its basename component).
+Defines the application name which will be pre-pended to all messages,
+followed by C<": "> (a colon and a space). Using this switch alone will
+configure the default driver to use that prefix (stripped down to its
+basename component).
 
-When a driver object is used, the C<-prefix> switch is kept at the Log::Agent
-level only and is not passed to the driver: it is up to the driver's creation
-routine to request the C<-prefix>. Having this information in Log::Agent
-enables the module to die on critical errors with that error prefix, since
-it cannot rely on the logging driver for that, obviously.
+When a driver object is used, the C<-prefix> switch is kept at the
+C<Log::Agent> level only and is not passed to the driver: it is up to
+the driver's creation routine to request the C<-prefix>. Having this
+information in Log::Agent enables the module to die on critical errors
+with that error prefix, since it cannot rely on the logging driver for
+that, obviously.
 
 =item C<-trace> => I<priority or level>
 
-Same a C<-debug> but for logtrc() calls only.
+Same a C<-debug> but applies to logsay(), logwarn(), logerr() and logtrc().
+
+When unspecified, C<Log::Agent> runs at the "notice" level.
 
 =back
 
 =head1 KNOWN LIMITATIONS
 
-The following limitations exist in this early version. They might be addressed
-in future versions if they are perceived as annoying limitatons instead of
-being just documented ones. :-)
+The following limitations exist in this early version. They might be
+addressed in future versions if they are perceived as annoying limitatons
+instead of being just documented ones. :-)
 
 =over
 
@@ -757,17 +612,17 @@ fix as one might think of at first glance.
 
 =item *
 
-Some drivers lack customization and hardwire a few things that come from
-my personal taste, like the prefixing done when I<duperr> is set in
-Log::Agent::Driver::File, or the fact that the C<debug> and C<output> channels
-are merged as one in the Log::Agent::Driver::Default driver.
+Some drivers lack customization and hardwire a few things that come
+from my personal taste, like the prefixing done when I<duperr> is set
+in Log::Agent::Driver::File, or the fact that the C<debug> and C<stderr>
+channels are merged as one in the Log::Agent::Driver::Default driver.
 
 =item *
 
-When using logcroak() or logconfess(), the place where the call was made
-can still be visible when -caller is used, since the addition of the
-caller information to the message is done before calling the logging driver.
-Is this a problem?
+When using logcroak() or logconfess(), the place where the call was
+made can still be visible when -caller is used, since the addition
+of the caller information to the message is done before calling the
+logging driver.  Is this a problem?
 
 =back
 
